/*
 * The MIT License
 *
 * Copyright 2016-2020 Charles University
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.cuni.mff.java.cv03;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Dynamic array
 */
public class DynArray implements MyCollection {

    private static final int PLUS = 10;

    private Object[] content = new Object[PLUS];
    private int count = 0;

    @Override
    public Iterator iterator() { // Iterator is a generic type but for now, we ignore the fact
        return new Iterator() {
            private int i = 0;

            @Override
            public boolean hasNext() {
                return i < count;
            }

            @Override
            public Object next() {
                if (i < count) {
                    return content[i++];
                }
                throw new NoSuchElementException();
            }
        };
    }

    @Override // this annotation causes compilation error if method is not actually overridden - ignore for now
    public void add(final Object o) { // it is a good practice to use `final` modifier for method parameters
        if (content.length == count) {
            // option 1 – better
            //content = Arrays.copyOf(content, content.length + PLUS);

            // option 2 – ideal when array which you are copying to is already created
            // Object[] newContent = new Object[content.length + PLUS];
            // System.arraycopy(content, 0, newContent, 0, content.length);
            // content = newContent;

            //option 3 – not ideal option
            Object[] newContent = new Object[content.length + PLUS];
            for (int i = 0; i < content.length; i++) {
                newContent[i] = content[i];
            }
            content = newContent;
        }
        content[count++] = o;
    }

    @Override
    public Object get(int i) {
        if (i >= 0 && i < count) {
            return content[i];
        }
        throw new IndexOutOfBoundsException();
    }

    @Override
    public void remove(Object o) {
        for (int i = 0; i < count; i++) {
            if (content[i] == o) {
                remove(i);
                break;
            }
        }
    }

    @Override
    public void remove(int i) {
        if (i >= 0 && i < count) {
            for (int j = i; j < count - 1; j++) {
                content[j] = content[j + 1];
            }
            count--;
        }
        throw new IndexOutOfBoundsException();
    }

    @Override
    public int size() {
        return count;
    }

}
